/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "lists.h"
#include "xml.h"

namespace mysdl
{
	namespace gui
	{
		// ================================================================
		//  ListItem
		// ================================================================

		/*
		 * See header file for documentation.
		 */
		bool ListItem::operator==(const ListItem &item)
		{
			return((caption==item.caption)&&(id==item.id)&&(data==item.data));
		};

		/*
		 * See header file for documentation.
		 */
		void ListItem::getFromNode(xml::Node *node)
		{
			caption=node->getAttribute(XML_GUI_CAPTION,caption);
			id=(int)(node->getAttributeLong(XML_GUI_ID,id));
			data=0;
		}

		// ================================================================
		//  List
		// ================================================================

		/*
		 * See header file for documentation.
		 */
		List::List() : useCount(0),list()
		{
		}

		/*
		 * See header file for documentation.
		 */
		List::~List()
		{
			clear();
		}

		/*
		 * See header file for documentation.
		 */
		void List::use()
		{
			useCount++;
		}

		/*
		 * See header file for documentation.
		 */
		void List::unuse()
		{
			//debug::addMessage("List::unuse()");

			useCount--;

			if(useCount<=0)
			{
				delete this;
			}

			//debug::removeMessage();
		}

		/*
		 * See header file for documentation.
		 */
		void List::clear()
		{
			//debug::addMessage("List::clear()");

			for(std::list<ListItem *>::iterator itr=list.begin(); itr!=list.end(); itr++)
			{
				delete *itr;
			}

			//debug::removeMessage();
		}

		/*
		 * See header file for documentation.
		 */
		void List::operator+=(ListItem *item)
		{
			//debug::addMessage("List::operator+=()");
			list.push_back(item);

			/*
			if(!selected)
			{
				selectItem(item);
			}
			*/

			//debug::removeMessage();
		}

		/*
		 * See header file for documentation.
		 */
		void List::operator-=(ListItem *item)
		{
			//debug::addMessage("List::operator-=()");
			list.remove(item);
			// \todo change selection if necessary
			//debug::removeMessage();
		}

		/*
		 * See header file for documentation.
		 */
		ListItem *List::findByData(void *data)
		{
			for(std::list<ListItem *>::iterator itr=list.begin(); itr!=list.end(); itr++)
			{
				if((*itr)->data==data)
				{
					return(*itr);
				}
			}

			return(0);
		}

		// ================================================================
		//  DropdownList
		// ================================================================

		/*
		 * See header file for documentation.
		 */
		DropdownList::DropdownList() : Component(),CaptionedObject(),holding(false),style(0),trigger(0),items(),selected(0)
		{
			wantsFocus=true;
		}

		/*
		 * See header file for documentation.
		 */
		DropdownList::~DropdownList()
		{
			//debug::addMessage("DropdownList::~DropdownList()");

			/*for(std::list<ListItem *>::iterator itr=items->list.begin(); itr!=items->list.end(); itr++)
			{
				delete *itr;
			}*/
			if(items)
			{
				items->unuse();
			}

			items=0;

			//debug::removeMessage();
		}

		/*
		 * See header file for documentation.
		 */
		void DropdownList::getSelfFromNode(const xml::Node *node,GUI *gui)
		{
			//debug::addMessage("DropdownList::getSelfFromNode()");

			Component::getSelfFromNode(node,gui);
			CaptionedObject::getSelfFromNode(node,gui);

			if(const std::string *style=node->getAttribute(XML_GUI_STYLE))
			{
				this->style=dynamic_cast<DropdownListStyle *>((*gui->getStyles())[*style]);

				if(this->style)
				{
					debug::debug(dbgGui,"Attached style \"%s\" to dropdown list \"%s\".",this->style->name.c_str(),name.c_str());
				}
				else
				{
					debug::warning(dbgGui,"Could not find matching style for dropdown list \"%s\".",name.c_str());
				}
			}

			if(const std::string *trigger=node->getAttribute(XML_GUI_TRIGGER))
			{
				this->trigger=((*gui->getTriggers())[*trigger]);
			}

			for(xml::Nodes::const_iterator itr=node->children.begin(); itr!=node->children.end(); itr++)
			{
				xml::Node *subNode=*itr;

				if(subNode->name==XML_GUI_ITEM)
				{
					if(!items)
					{
						items=new List();
						items->use();
					}

					ListItem *item=new ListItem();
					item->getFromNode(subNode);
					(*items)+=item;
				}
			}

			if(items)
			{
				setItems(items);
			}

			//debug::removeMessage();
		}

		/*
		 * See header file for documentation.
		 */
		void DropdownList::setItems(List *items)
		{
			if(this->items!=items)
			{
				if(this->items)
				{
					this->items->unuse();
				}

				this->items=items;

				if(items)
				{
					items->use();
				}
			}

			if(items)
			{
				if(items->list.empty())
				{
					selectItem(0);
				}
				else
				{
					if(!selected)
					{
						selectItem(items->list.front());
					}
					else
					{
						ListItem *newItem=items->list.front();

						for(std::list<ListItem *>::iterator itr=items->list.begin(); itr!=items->list.end(); itr++)
						{
							if(*itr==selected)
							{
								newItem=selected;
							}
						}

						selectItem(newItem);
					}
				}
			}
		}

		/*
		 * See header file for documentation.
		 */
		void DropdownList::selectItem(ListItem *item)
		{
			selected=item;

			if(item)
			{
				caption=item->caption;
				debug::debug(dbgDdd,"Selected dropdown list item \"%s\".",caption.c_str());
			}
			else
			{
				caption="";
				debug::debug(dbgDdd,"Selected dropdown list item null.");
			}
		}

		/*
		 * See header file for documentation.
		 */
		void DropdownList::drawSelf()
		{
			//debug::addMessage("DropdownList::drawSelf()");

			if(style)
			{
				ButtonState state=getState();

				mysdl::Rect rect;
				rect.pos=Vector2D(0,0);
				rect.dimensions=dimensions;
				style->draw(rect,state);

				glPushMatrix();
				style->print(rect,state,caption.c_str());
				glPopMatrix();
			}

			//debug::removeMessage();
		}

		/*
		 * See header file for documentation.
		 */
		void DropdownList::mouseDown(ControlQueue *queue,Vector2D pos,Uint8 button)
		{
			if(button==1)
			{
				captureMouse(true);
				holding=true;
			}
		}

		/*
		 * See header file for documentation.
		 */
		void DropdownList::mouseUp(ControlQueue *queue,Vector2D pos,Uint8 button)
		{
			if(button==1)
			{
				if(isDown())
				{
					doClick(queue);
				}

				holding=false;
				captureMouse(false);
			}
		}

		/*
		 * See header file for documentation.
		 */
		void DropdownList::doClick(ControlQueue *queue)
		{
			if(items)
			{
				if(!items->list.empty())
				{
					if(selected)
					{
						std::list<ListItem *>::iterator itr;

						for(itr=items->list.begin(); itr!=items->list.end(); itr++)
						{
							if((*itr)==selected)
							{
								break;
							}
						}

						itr++;

						if(itr==items->list.end())
						{
							selectItem(items->list.front());
						}
						else
						{
							selectItem(*itr);
						}
					}
					else
					{
						selectItem(items->list.front());
					}
				}
			}

			if(trigger)
			{
				(*queue)<<trigger;
			}
		}

		/*
		 * See header file for documentation.
		 */
		bool DropdownList::keyDown(ControlQueue *queue,SDLKey key,SDLMod modifier,Uint16 unicode)
		{
			if((key==SDLK_RETURN)||(key==SDLK_SPACE))
			{
				doClick(queue);
				return(true);
			}
			else
			{
				return(Component::keyDown(queue,key,modifier,unicode));
			}
		}

		/*
		 * See header file for documentation.
		 */
		ButtonState DropdownList::getState()
		{
			ButtonState state=ButtonState(0);

			if(isDown())
			{
				state=ButtonState(state | bsDown);
			}

			if(focused)
			{
				state=ButtonState(state | bsHot);
			}

			return(state);
		}

		/*
		 * See header file for documentation.
		 */
		bool DropdownList::isDown()
		{
			return((holding)&&(focused));
		}

		/*
		 * See header file for documentation.
		 */
		List *DropdownList::getItems()
		{
			return(items);
		}

		/*
		 * See header file for documentation.
		 */
		ListItem *DropdownList::getSelected()
		{
			return(selected);
		}

		/*
		 * See header file for documentation.
		 */
		void DropdownList::setStyle(DropdownListStyle *style)
		{
			this->style=style;
		}

		/*
		 * See header file for documentation.
		 */
		DropdownListStyle *DropdownList::getStyle()
		{
			return(style);
		}

		/*
		 * See header file for documentation.
		 */
		void DropdownList::setTrigger(Trigger *trigger)
		{
			this->trigger=trigger;
		}

		/*
		 * See header file for documentation.
		 */
		Trigger *DropdownList::getTrigger()
		{
			return(trigger);
		}
	};
};
