/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "explosion.h"

namespace mysdl
{
	namespace particles
	{
		Explosion::Explosion(ExplosionProperties properties) :
			properties(properties),pos(),particles(0),particleCount(0),drawCount(0)
		{
			init();
		}

		Explosion::~Explosion()
		{
			release();
		}

		void Explosion::release()
		{
			if(particles)
			{
				delete[] particles;
				delete[] array;
				particles=0;
			}

			particleCount=0;
			drawCount=0;
		}

		void Explosion::init()
		{
			release();

			particleCount=properties.count;

			array=new GLfloat[particleCount*32];//ExplosionParticleEntry[particleCount];
			particles=new ExplosionParticle[particleCount];

			for(Uint32 i=0; i<particleCount; i++)
			{
				ExplosionParticle &p=particles[i];

				GLfloat a=random<GLfloat>(3.141592654*2);
				GLfloat r=random<GLfloat>(1.0);

				GLfloat x=cos(a)*r*properties.speedFactor.x;
				GLfloat y=sin(a)*r*properties.speedFactor.y;

				GLfloat c=random<GLfloat>(0.0005,0.001);

				p.pos=Vector2D(0.0,0.0);
				p.color=Color(1.0,1.0,random<GLfloat>(0.2),1.0);
				p.velocity=Vector2D(x,y);
				p.colorVelocity=Color(0.0,-c,0,-c);
			}
		};

		void Explosion::perform(GLfloat ticks)
		{
			drawCount=0;

			for(Uint32 i=0; i<particleCount; i++)
			{
				ExplosionParticle &p=particles[i];

				p.perform(ticks);//,this);

				if(p.active)
				{
					Vector2D pp=pos+p.pos;

					/*
					ExplosionParticleEntry &e=array[drawCount];

					e.edge1.x=pp.x-properties.radius;
					e.edge1.y=pp.y-properties.radius;

					e.edge2.x=pp.x+properties.radius;
					e.edge2.y=pp.y-properties.radius;

					e.edge3.x=pp.x+properties.radius;
					e.edge3.y=pp.y+properties.radius;

					e.edge4.x=pp.x-properties.radius;
					e.edge4.y=pp.y+properties.radius;

					e.color1=e.color2=e.color3=e.color4=p.color;
					*/

					GLfloat *a=&(array[drawCount*32]);

					a[ 0]=(GLint)(pp.x-properties.radius);
					a[ 1]=(GLint)(pp.y-properties.radius);
					a[ 2]=0;
					a[ 3]=1;

					a[ 4]=p.color.r;
					a[ 5]=p.color.g;
					a[ 6]=p.color.b;
					a[ 7]=p.color.a;

					a[ 8]=(GLint)(pp.x+properties.radius);
					a[ 9]=(GLint)(pp.y-properties.radius);
					a[10]=1;
					a[11]=1;

					a[12]=p.color.r;
					a[13]=p.color.g;
					a[14]=p.color.b;
					a[15]=p.color.a;

					a[16]=(GLint)(pp.x+properties.radius);
					a[17]=(GLint)(pp.y+properties.radius);
					a[18]=1;
					a[19]=0;

					a[20]=p.color.r;
					a[21]=p.color.g;
					a[22]=p.color.b;
					a[23]=p.color.a;

					a[24]=(GLint)(pp.x-properties.radius);
					a[25]=(GLint)(pp.y+properties.radius);
					a[26]=0;
					a[27]=0;

					a[28]=p.color.r;
					a[29]=p.color.g;
					a[30]=p.color.b;
					a[31]=p.color.a;

					drawCount++;
				}
			}
		}

		void Explosion::draw()
		{
			glPushAttrib(GL_TEXTURE_BIT|GL_COLOR_BUFFER_BIT);
			glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT);
			glPushMatrix();

			GLuint tex=dynamic_cast<mysdl::gfx::SingleTextureImage *>(properties.particleImage)->getTextureName();

			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA,GL_ONE);
			glEnable(GL_TEXTURE_2D);
			//glDisable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D,tex); //\todo make better

			glVertexPointer(2,GL_FLOAT,sizeof(GLfloat)*8,&(array[0]));
			glTexCoordPointer(2,GL_FLOAT,sizeof(GLfloat)*8,&(array[2]));
			glColorPointer(4,GL_FLOAT,sizeof(GLfloat)*8,&(array[4]));

			glEnableClientState(GL_VERTEX_ARRAY);
			glEnableClientState(GL_COLOR_ARRAY);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);

			glDrawArrays(GL_QUADS,0,drawCount*4);
/*
			glBegin(GL_QUADS);

			for(Uint32 i=0; i<drawCount*4; i++)
			{
				glColor4fv(&(array[i*8+4]));
				glTexCoord2fv(&(array[i*8+2]));
				glVertex2fv(&(array[i*8+0]));
			}

			glEnd();
*/

			glPopMatrix();
			glPopClientAttrib();
			glPopAttrib();
		}

		Explosion *ExplosionProperties::create() const
		{
			return(new Explosion(*this));
		}
	};
};
