/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "starfield.h"

namespace mysdl
{
	namespace particles
	{
		Starfield *StarfieldProperties::create()
		{
			return(new Starfield(*this));
		}
		
		Starfield::Starfield(StarfieldProperties properties) :
			properties(properties),stars(0),starCount(0),speedFactor(1)
		{
			init();
		}
		
		Starfield::~Starfield()
		{
			release();
		}
		
		void Starfield::release()
		{
			if(stars)
			{
				delete[] stars;
				stars=0;
			}
			
			starCount=0;
		}
		
		void Starfield::init()
		{
			release();
			
			for(std::vector<StarClass>::iterator itr=properties.classes.begin(); itr!=properties.classes.end(); itr++)
			{
				starCount+=itr->count;
			}
			
			stars=new Star[starCount];
			
			Uint32 oldStarCount=0;
			
			for(std::vector<StarClass>::iterator itr=properties.classes.begin(); itr!=properties.classes.end(); itr++)
			{
				Uint32 newStarCount=oldStarCount+itr->count;
				
				for(Uint32 i=oldStarCount; i<newStarCount; i++)
				{
					stars[i].pos.x=random<GLfloat>(properties.dimensions.x);
					stars[i].pos.y=random<GLfloat>(properties.dimensions.y);
				}
				
				oldStarCount=newStarCount;
			}
		}
		
		void Starfield::perform(GLfloat factor)
		{
			factor*=speedFactor;
			
			Uint32 prevIndex=0;
			
			for(std::vector<StarClass>::iterator itr=properties.classes.begin(); itr!=properties.classes.end(); itr++)
			{
				Uint32 newIndex=prevIndex+itr->count;
				
				for(Uint32 i=prevIndex; i<newIndex; i++)
				{
					stars[i].pos.y-=itr->speed*factor;
					
					if(stars[i].pos.y<0)
					{
						stars[i].pos.x=random<GLfloat>(0,properties.dimensions.x);
						stars[i].pos.y+=properties.dimensions.y;
					}
				}
				
				prevIndex=newIndex;
			}
		}
		
		void Starfield::draw()
		{
			glPushAttrib(GL_TEXTURE_BIT|GL_COLOR_BUFFER_BIT);
			glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT);
			glPushMatrix();
			
			glDisable(GL_BLEND);
			glDisable(GL_TEXTURE_2D);
			glVertexPointer(2,GL_FLOAT,0,stars);
			glEnableClientState(GL_VERTEX_ARRAY);
			
			Uint32 index=0;
			
			for(std::vector<StarClass>::iterator itr=properties.classes.begin(); itr!=properties.classes.end(); itr++)
			{
				itr->color.apply();
				glDrawArrays(GL_POINTS,index,itr->count);
				index+=itr->count;
			}
			
			glPopMatrix();
			glPopClientAttrib();
			glPopAttrib();
		}
	};
};
